@@ -178,6 +178,170 @@ class GenericReader:
 			return self.read_uint()
 			#raw_data = self.read(pos, self.sizeof_long)
 			#return struct.unpack(self.unpack_long, raw_data)[0]
+
+class AGenericReader:
+	def __init__(self, data, processor_architecture = KatzSystemArchitecture.X64):
+		"""
+		data is bytes
+		"""	
+		self.processor_architecture = processor_architecture
+		self.start_address = 0
+		self.end_address = len(data)
+		self.size = len(data)
+		self.data = data
+		self.current_position = 0
+		
+	def inrange(self, ptr):
+		return self.start_address <= ptr <= self.end_address
+		
+	async def seek(self, offset, whence = 0):
+		"""
+		Changes the current address to an offset of offset. The whence parameter controls from which position should we count the offsets.
+		0: beginning of the current memory segment
+		1: from current position
+		2: from the end of the current memory segment
+		If you wish to move out from the segment, use the 'move' function
+		"""
+		if whence == 0:
+			t = self.start_address + offset
+		elif whence == 1:
+			t = self.current_position + offset
+		elif whence == 2:
+			t = self.end_address - offset
+		else:
+			raise Exception('Seek function whence value must be between 0-2')
+			
+		if not self.inrange(t):
+			raise Exception('Seek would point out of buffer')
+		
+		self.current_position = t
+		return
+		
+	async def move(self, address):
+		"""
+		Moves the buffer to a virtual address specified by address
+		"""
+		await self.seek(address)
+		return
+		
+	async def align(self, alignment = None):
+		"""
+		Repositions the current reader to match architecture alignment
+		"""
+		if alignment is None:
+			if self.processor_architecture == KatzSystemArchitecture.X64:
+				alignment = 8
+			else:
+				alignment = 4
+		offset = self.current_position % alignment
+		if offset == 0:
+			return
+		offset_to_aligned = (alignment - offset) % alignment
+		await self.seek(offset_to_aligned, 1)
+		return
+		
+	def tell(self):
+		"""
+		Returns the current virtual address
+		"""
+		return self.current_position
+		
+	async def peek(self, length):
+		"""
+		Returns up to length bytes from the current memory segment
+		"""
+		t = self.current_position + length
+		if not self.inrange(t):
+			raise Exception('Would read out of buffer!')
+		return self.data[self.current_position - self.start_address :t - self.start_address]
+	
+	async def read(self, size = -1):
+		"""
+		Returns data bytes of size size from the current segment. If size is -1 it returns all the remaining data bytes from memory segment
+		"""
+		if size < -1:
+			raise Exception('You shouldnt be doing this')
+		if size == -1:
+			oldnew_pos = self.current_position
+			self.current_position = self.end_address
+			return self.data[oldnew_pos:]
+		
+		t = self.current_position + size
+		if not self.inrange(t):
+			raise Exception('Would read out of buffer!')
+		
+		old_new_pos = self.current_position
+		self.current_position = t		
+		return self.data[old_new_pos:t]
+	
+	async def read_int(self):
+		"""
+		Reads an integer. The size depends on the architecture. 
+		Reads a 4 byte small-endian singed int on 32 bit arch
+		Reads an 8 byte small-endian singed int on 64 bit arch
+		"""
+		if self.processor_architecture == KatzSystemArchitecture.X64:
+			t = await self.read(8)
+			return int.from_bytes(t, byteorder = 'little', signed = True)
+		else:
+			t = await self.read(4)
+			return int.from_bytes(t, byteorder = 'little', signed = True)
+	
+	async def read_uint(self):
+		"""
+		Reads an integer. The size depends on the architecture. 
+		Reads a 4 byte small-endian unsinged int on 32 bit arch
+		Reads an 8 byte small-endian unsinged int on 64 bit arch
+		"""
+		if self.processor_architecture == KatzSystemArchitecture.X64:
+			t = await self.read(8)
+			return int.from_bytes(t, byteorder = 'little', signed = False)
+		else:
+			t = await self.read(4)
+			return int.from_bytes(t, byteorder = 'little', signed = False)
+	
+	async def find(self, pattern):
+		"""
+		Searches for a pattern in the current memory segment
+		"""
+		pos = await self.data.find(pattern)
+		if pos == -1:
+			return -1
+		return pos + self.current_position
+		
+	async def find_all(self, pattern):
+		"""
+		Searches for all occurrences of a pattern in the current memory segment, returns all occurrences as a list
+		"""
+		pos = []
+		last_found = -1
+		while True:
+			last_found = await self.data.find(pattern, last_found + 1)
+			if last_found == -1:
+				break
+			pos.append(last_found + self.start_address)
+			
+		return pos
+		
+	async def get_ptr(self, pos):
+		await self.move(pos)
+		return await self.read_uint()
+		#raw_data = self.read(pos, self.sizeof_ptr)
+		#return struct.unpack(self.unpack_ptr, raw_data)[0]
+	
+	async def get_ptr_with_offset(self, pos):
+		if self.processor_architecture == KatzSystemArchitecture.X64:
+			await self.move(pos)
+			ptr = int.from_bytes(self.read(4), byteorder = 'little', signed = True)
+			return pos + 4 + ptr
+			#raw_data = self.read(pos, self.sizeof_long)
+			#ptr = struct.unpack(self.unpack_long, raw_data)[0]
+			#return pos + self.sizeof_long + ptr
+		else:
+			await self.move(pos)
+			return await self.read_uint()
+			#raw_data = self.read(pos, self.sizeof_long)
+			#return struct.unpack(self.unpack_long, raw_data)[0]
 		
 		
 		
