@@ -55,7 +55,12 @@ class LiveSspDecryptor(PackageDecryptor):
 		c.domainname = suppCreds.credentials.Domaine.read_string(self.reader)
 		if suppCreds.credentials.Password.Length != 0:
 			enc_data = suppCreds.credentials.Password.read_maxdata(self.reader)
-			c.password = self.decrypt_password(enc_data)
+			if c.username.endswith('$') is True:
+				c.password = self.decrypt_password(enc_data, bytes_expected=True)
+				if c.password is not None:
+					c.password = c.password.hex()
+			else:
+				c.password = self.decrypt_password(enc_data)
 		
 		self.credentials.append(c)
 	
